/***************************************************************************
                          hi_score.c
  description: introduction, hi score and score sheet
                             -------------------
    begin                : Fri March 30 2001
    date                 : 27/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <SDL/SDL.h>
#include "structs.h" //global screen
#include "data.h" // LoadImage()
#include "sprite.h"
#include "defs.h"
#include "hi_score.h"

/* HiScore display. */
#define HS_ALPHA_X  90  // x offset of initials
#define HS_DIGIT_X  300 // x offset of score
#define HS_Y        80  // y offset
#define HS_FONT_W   35  // 'font' width
#define HS_FONT_H   40  // 'font' height

SDL_Surface *backgroundHiScore;

SDL_Surface *digit_30_image;
SDL_Surface *alpha_30_image;


int hi_score_init(void)
{
	if ((backgroundHiScore = LoadImage(DATAFILE("backgroundHiScore.gif"), 0)) == NULL)
		return -1;
	if ((digit_30_image = LoadImage(DATAFILE("digit_30_flame.gif"), 0)) == NULL)
		return -1;
	if ((alpha_30_image = LoadImage(DATAFILE("alpha_30_green.gif"), 0)) == NULL)
		return -1;
	return 1;
}


void hi_score_free(void)
{
	SDL_FreeSurface(backgroundHiScore);
	SDL_FreeSurface(digit_30_image);
	SDL_FreeSurface(alpha_30_image);
}


/**
 * hi_score_read()
 * read the hiscore file
 *
 * Returns:     a newly allocated buffer of the hiscore chars or NULL.
 *              The buffer should be freed by the caller.
 *
 **/
char *hi_score_read(void)
{
	FILE *file;
	char *buf;
	size_t bytes_read = 0;

	buf = malloc(HS_MAX_CHARS);
	if (!buf)
		return NULL;

	file = fopen(DSI_HISCORE, "r+");
	if (!file)
	{
		printf("Cannot open hiscore file.\n");
		goto errout;
	}
	bytes_read = fread(buf, 1, HS_MAX_CHARS, file);
	fclose(file);

	if (bytes_read != HS_MAX_CHARS)
		goto errout;

	return buf;
errout:
	free(buf);
	return NULL;
}


/**
 * hi_score_write()
 * write the hiscore file
 *
 * @buf:        the buffer to write to file.
 *
 **/
void hi_score_write(char *buf)
{
	FILE *file;

	if ((file = fopen(DSI_HISCORE, "r+")) == NULL)
	{
		printf("Cannot open hiscore file.\n");
		return;
	}
	fwrite(buf, HS_MAX_CHARS, 1, file);
	fclose(file);
}


/**
 * hi_score_view()
 * view the hiscore file
 *
 * Returns:     a new run state determined by user input.
 *
 **/
int hi_score_view(void)
{
	static SDL_Rect srect = {0, 0, 30, 30};
	SDL_Rect drect;
	char *buf;
	int i, j;
	int leave_state = RUN_SCORE;
	int countdown = 5000;

	//Read the score sheet
	buf = hi_score_read();
	if (!buf)
		return RUN_INTRO;

	//Paint the background
	drect.x = 0;
	drect.y = 0;
	SDL_BlitSurface(backgroundHiScore, NULL, screen, &drect);

	//Create the hi-scores and initials
	for(i = 0; i < HS_MAX_CHARS; ++i)
	{
		int c;

		if (i < HS_MAX_DIGITS)
		{
			c = buf[i] - '0';
			if (c >= 0 && c <= 9)
			{
				drect.x = HS_DIGIT_X + ((i % MAX_DIGITS) * HS_FONT_W);
				drect.y = HS_Y + ((i / MAX_DIGITS) * HS_FONT_H);
				srect.x = c * 30;
				SDL_BlitSurface(digit_30_image, &srect, screen, &drect);
			}
		}
		else
		{
			j = i - HS_MAX_DIGITS;
			c = buf[i] - 'a';
			if (c >= 0 && c <= 'z' - 'a')
			{
				drect.x = HS_ALPHA_X + ((j % HS_MAX_INITIALS) * HS_FONT_W);
				drect.y = HS_Y + ((j / HS_MAX_INITIALS) * HS_FONT_H);
				srect.x = c * 30;
				SDL_BlitSurface(alpha_30_image, &srect, screen, &drect);
			}
		}
	}
	free(buf);
	SDL_UpdateRect(screen, 0, 0, 0, 0);

	while (leave_state == RUN_SCORE)
	{
		Uint8 *key;
		SDL_Event event;

		//countdown to return to main screen
		countdown = countdown - 1;
		if (!countdown)
			leave_state = RUN_INTRO;

		//Poll input queue, run keyboard loop
		while (SDL_PollEvent(&event))
		{
			if ( event.type == SDL_QUIT )
				leave_state = RUN_QUIT;
		}
		key = SDL_GetKeyState(NULL);
		if (key[SDLK_b] == SDL_PRESSED)
			leave_state = RUN_INTRO;
		else if (key[SDLK_s] == SDL_PRESSED)
			leave_state = RUN_GAME;
		WaitFrame();
	}
	return leave_state;
}
