/***************************************************************************
                           shots.c
    description          : shot handling
                           -------------------
    begin                : Fri March 30 2001, from rungame.c
    date                 : 01/08/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <string.h>
#include "aliens.h"
#include "data.h"
#include "structs.h" //global screen
#include "shots.h"

#define MAX_SHOTS       10
#define SHOT_SPEED      5


static Sprite shots[MAX_SHOTS];
static Uint32 shot_time = 50; //default: approx 1 second.
static Uint32 timeout = 0;


/**
 *  shots_init()
 *  initializes shot data
 *
 * Returns:     -1 on error, otherwise count of Sprites.
 *
**/
int shots_init(void)
{
	int i;

	memset(&shots, 0, MAX_SHOTS * sizeof(Sprite));
	if ((shots[0].image = LoadImage(DATAFILE("shot.gif"), 0)) == NULL)
		return -1;
	for (i = 1; i < MAX_SHOTS; ++i)
		shots[i].image = shots[0].image;
	return MAX_SHOTS;
}


/**
 *  shots_free()
 *  frees shot data
 *
**/
void shots_free(void)
{
	if (shots[0].image)
		SDL_FreeSurface(shots[0].image);
}


/**
 *  shots_set_timer()
 *  set min time between shots, clamped 2 to 50.
 *
 *  @count:      counter.
 *
**/
void shots_set_timer(int count)
{
	if (count < 2)
		count = 2;
	if (count > 50)
		count = 50;
	shot_time = count;
}



/**
 *  shots_add()
 *  find a free shot slot, if shot delay has timed out.
 *
 *  Returns:    pointer to a free shot Sprite, or NULL.
 *
**/
Sprite *shots_add(void)
{
	int i;

	if (!timeout)
	{
		for (i = MAX_SHOTS - 1; i >= 0; --i)
		{
			if (!shots[i].alive)
			{
				//reset shot "timer"
				timeout = shot_time;
				return &shots[i];
			}
		}
	}
	return NULL;
}

/**
 *  shots_move()
 *  move all shots up, kill if at heads-up
 *
**/
void shots_move(void)
{
	int i;

	if (timeout)
		--timeout;
	for (i = MAX_SHOTS - 1; i >= 0; --i)
	{
		if (shots[i].alive)
		{
			shots[i].y -= SHOT_SPEED;
			if ( shots[i].y < HU_Y + HU_H ) //shots die below heads-up line
				shots[i].alive = 0;
		}
	}
}


/**
 *  shots_collide()
 *  check all shots for collision with aliens, kill shot if collided.
 *
 *  Returns:    1 on collision, else 0.
 *
**/
int shots_collide(void)
{
	int i;
	int points = 0;

	for (i = MAX_SHOTS - 1; i >= 0; --i)
	{
		if (shots[i].alive)
		{
			int hit = aliens_collide(&shots[i], 1);
			if (hit)
			{
				shots[i].alive = 0;
				points += hit;
			}
		}
	}
	return points;
}


/**
 *  shots_foreach()
 *  apply function to all shots.
 *
 *  @dofunc:    function to apply.
 *
**/
void shots_foreach(SpriteFunc dofunc)
{
	int i;

	for (i = MAX_SHOTS - 1; i >= 0; --i)
	{
		if (shots[i].alive)
			dofunc(&shots[i]);
	}
}
